#ifndef _MCDQHULLTYPES_H /* -*- mode: C; -*- */
#define _MCDQHULLTYPES_H

/*
   Copyright (c) 1997-2002 MathEngine PLC

   $Name:  $

   Date: $Date: 2003/05/05 00:57:07 $ - Revision: $Revision: 1.1 $

   This software and its accompanying manuals have been developed
   by MathEngine PLC ("MathEngine") and the copyright and all other
   intellectual property rights in them belong to MathEngine. All
   rights conferred by law (including rights under international
   copyright conventions) are reserved to MathEngine. This software
   may also incorporate information which is confidential to
   MathEngine.

   Save to the extent permitted by law, or as otherwise expressly
   permitted by MathEngine, this software and the manuals must not
   be copied (in whole or in part), re-arranged, altered or adapted
   in any way without the prior written consent of the Company. In
   addition, the information contained in the software may not be
   disseminated without the prior written consent of MathEngine.

 */
/** @file */

#include <MeMath.h>
#include <McdGeometryTypes.h>

#ifdef __cplusplus
extern "C" {
#endif 

/**
  Create new convex polyhedron object from its vertices. Computes
  the convex hull polygons and edges.

  return 0 on failure
*/
MEPUBLIC
McdConvexMeshID   MEAPI McdConvexMeshCreateHull(McdFramework *frame,
                            const MeVector3 *vertices,
                            int vertexCount, MeReal fatnessRadius );

/**
  This takes some points and returns an McdConvexHull structure. The
  memory for the convex hull data is allocated with MeMemoryAPI.create.

  returns 1 on success, or 0 on error.
*/
MEPUBLIC
int McdComputeHull(McdConvexHull *cnv,
                   int numpoints, const MeVector3 *points);

/**
  This computes the convex hull and returns the number of vertices,
  edges, and faces.

  It is presumed that the caller will invoke this function to
  determine the memory required for the convex hull, then allocate the 
  memory, and then call McdGetHullData to get the actual data.

  returns 1 on success, or 0 on error.
*/
int McdComputeHullSizes(McdConvexHull *cnv,
                        int numpoints, const MeVector3 *points);

/**
  This gets the convex hull data that was generated in the previous call
  to McdComputeHullSizes.

  The caller must allocate the arrays for faces, vertices, edges and 
  edge index.  NOTE!! the face and vertex array must have room for one
  extra for the end sentinel.

  returns 1 on success, 0 on failure.
*/
int McdGetHullData(McdConvexHull *cnv);


/**
  Create new convex hull from a list of planes.

  Input is a list of Vec4.  Each vec4 is a outward pointing unit normal 
  vector and a distance.

  returns 1 on success, or 0 on error.
*/
int MEAPI 
McdComputeHullFromPlanes(McdConvexHull *h, int planeCount,
                         const MeVector4 *planes);

/**
  Create new convex hull from a list of planes.

  Input is a list of Vec4.  Each vec4 is a outward pointing unit normal 
  vector and a distance.

  returns 0 on error.
*/
McdConvexMeshID MEAPI 
McdConvexMeshCreateFromPlanes(McdFramework *frame,
                              const MeVector4 *planes,
                              int planeCount, MeReal fatness );


#ifdef __cplusplus
}
#endif 

#endif
