#ifndef _MDTARTHUR_H /* -*- mode: C; -*- */
#define _MDTARTHUR_H

/*
   Copyright (c) 2001 MathEngine PLC

   $Name:  $

   Date: $Date: 2003/05/05 00:57:07 $ - Revision: $Revision: 1.1 $

   This software and its accompanying manuals have been developed
   by MathEngine PLC ("MathEngine") and the copyright and all other
   intellectual property rights in them belong to MathEngine. All
   rights conferred by law (including rights under international
   copyright conventions) are reserved to MathEngine. This software
   may also incorporate information which is confidential to
   MathEngine.

   Save to the extent permitted by law, or as otherwise expressly
   permitted by MathEngine, this software and the manuals must not
   be copied (in whole or in part), re-arranged, altered or adapted
   in any way without the prior written consent of the Company. In
   addition, the information contained in the software may not be
   disseminated without the prior written consent of MathEngine.

 */
#include <MePrecision.h>
#include <MdtSolverUtils.h>
#include <MeChunk.h>
#include <MdtTypes.h>

typedef struct 
{
    MeVector4 col[6];
} MdtArthurJBlock;

typedef struct 
{
    MdtArthurJBlock *  block;
    int                numElts; 
} MdtArthurJBlockArray;

typedef struct
{
    MdtArthurJBlock * block;               /* Array of length numBlocks */
    MeI32Array partitionedBlocksPerBody;   /* Array of length numBodies */
    MeI32Array partitionedBodyToIndex;     /* Array of length numBodies */
    MeI32Array partitionedBlockToConstRow; /* Array of length numBlocks */
    MeI32Array blockToConstRow;            /* Array of length numBlocks */
    MeI32Array constRowToBlock;            /* Array of length numConstBlocks*numBodies */
	MeI32Array bodyToIndex;                /* Array of length numBodies */
	MeI32Array blocksPerBody;              /* Array of length numBodies */
    MeI32Array blocksPerPartition;         /* Array of length numPartitions */
    MeI32Array bodiesPerPartition;         /* Array of length numPartitions */
    int numBodies;
	int numBlocks;
    int numConstBlocks;

} MdtArthurJM;

typedef struct
{
    int firstBlockInBin[4];  /* PS2 only, first body in each of the 4 SPR bins */
    int numBins;
} MdtArthurJMBinInfo;

typedef struct {

    MdtArthurJM             JM;
    MeVector4N              rhs;
    MeVector4N              vel;
    MeVector4N              invMdiagonal;
    MeVector4N              lomask;
    MeVector4N              himask;

} MdtArthurInterData;

#ifdef __cplusplus
extern "C"
{
#endif

#define MAX_ROWS_INC_PADDING 340

typedef struct
{
    MeVector4 vel[MAX_ROWS_INC_PADDING/4];
    MeVector4 lomask[MAX_ROWS_INC_PADDING/4];
    MeVector4 himask[MAX_ROWS_INC_PADDING/4];
    
} VUMEM_JMf;

MEPUBLIC
void MEAPI MdtArthurCalcConstraintForces(
               MdtSolverOutputData        outputData,   /* Output */ 
               MdtArthurInterData         interData,    /* Output */
               const MdtSolverInputData   inputData);   /* Input */

void MEAPI convertKeaVectorsToArthurVectors(
                                      MeVector4N lo,               /* Output */
                                      MeVector4N hi,               /* Output */
                                      MeVector4N slipfactor,       /* Output */
                                      MeVector4N xgamma,           /* Output */
                                      MeVector4N c,                /* Output */
                                      MeVector4N xi,               /* Output */
                                      const MeReal              kxgamma[],         /* Input */
                                      const MeReal              kc[],              /* Input */
                                      const MeReal              kxi[],             /* Input */
                                      const MeReal              keaLo[],          /* Input */
                                      const MeReal              keaHi[],          /* Input */
                                      const MeReal              keaSlipfactor[],  /* Input */
                                      const int                 Jsize[],          /* Input */
                                      const int                 numConstraints[], /* Input */
                                      int                       numPartitions);   /* Input */

void MEAPI convertKeaJtoMdtVelocityJacobian(
                           MdtVelocityJacobian       J,                /* Output */
                           const MdtKeaJBlockPair    Jstore[],         /* Input */
                           const int                 numConstraints[], /* Input */
                           const int                 Jsize[],          /* Input */
                           const MeI32PairArray      Jbody,            /* Input */
                           int                       numPartitions);   /* Input */

void MEAPI makeBodyForcesPointerArray(
                           MeLinAngPtrArray          externalForce,  /* Output */
                           MeLinAngPtrArray          resultantForce, /* Output */
                           MdtKeaBody *const         blist[],        /* Input */ 
                           int                       numBodies);     /* Input */

#ifdef __cplusplus
}
#endif

#endif
