#ifndef _MDTCONTACT_H
#define _MDTCONTACT_H
/* -*- mode: C; -*- */

/*
   Copyright (c) 1997-2002 MathEngine PLC

   $Name:  $

   Date: $Date: 2003/05/05 00:57:07 $ - Revision: $Revision: 1.1 $

   This software and its accompanying manuals have been developed
   by MathEngine PLC ("MathEngine") and the copyright and all other
   intellectual property rights in them belong to MathEngine. All
   rights conferred by law (including rights under international
   copyright conventions) are reserved to MathEngine. This software
   may also incorporate information which is confidential to
   MathEngine.

   Save to the extent permitted by law, or as otherwise expressly
   permitted by MathEngine, this software and the manuals must not
   be copied (in whole or in part), re-arranged, altered or adapted
   in any way without the prior written consent of the Company. In
   addition, the information contained in the software may not be
   disseminated without the prior written consent of MathEngine.

 */

/** @file
 * MdtContact API functions.
 */

#include <MePrecision.h>
#include <MdtTypes.h>

#ifdef __cplusplus
extern "C"
{
#endif



/*
  Contact functions.
*/

MEPUBLIC
MdtContactID      MEAPI MdtContactCreate(const MdtContactGroupID group);
MEPUBLIC
MdtContactID      MEAPI MdtContactInit(MdtContact *const c,
                                       const MdtContactGroupID group);

void              MEAPI MdtContactDestroy(const MdtContactID c);
void              MEAPI MdtContactTerm(const MdtContactID c);
/*
  Contact accessors.
*/

MEPUBLIC
void              MEAPI MdtContactGetPosition(const MdtContactID c, MeVector3 v);
MEPUBLIC
void              MEAPI MdtContactGetNormal(const MdtContactID c, MeVector3 v);
MEPUBLIC
MeReal            MEAPI MdtContactGetPenetration(const MdtContactID c);
MEPUBLIC
void              MEAPI MdtContactGetDirection(const MdtContactID c, MeVector3 v);
MEPUBLIC
void              MEAPI MdtContactGetWorldVelocity(const MdtContactID c, MeVector3 v);
MEPUBLIC
MdtContactParamsID 
                  MEAPI MdtContactGetParams(const MdtContactID c);
MEPUBLIC
MeI32             MEAPI MdtContactGetDimensions(const MdtContactID c); 

MEPUBLIC
MdtContactID      MEAPI MdtContactGetNext(const MdtContactID c); /* DEPRECATED!! */

MEPUBLIC
MdtContactGroupID MEAPI MdtContactGetContactGroup(const MdtContactID c);
MEPUBLIC
void              MEAPI MdtContactGetRelativeVelocity(const MdtContactID c, 
                                                      MdtBodyID b0, 
                                                      MdtBodyID b1, 
                                                      MeVector3 v);
MEPUBLIC
void *            MEAPI MdtContactGetUserData(const MdtContactID c); /* DEPRECATED!! */


/** Invalid ContactID indicating end of MdtContact linked list. */
extern const MdtContactID MdtContactInvalidID;

void              MEAPI MdtContactGetForce(const MdtContactID c, int bodyindex, MeVector3 force);
void              MEAPI MdtContactGetTorque(const MdtContactID c, int bodyindex, MeVector3 torque);


/*
  Contact mutators.
*/

MEPUBLIC
void              MEAPI MdtContactSetPosition(const MdtContactID c,
                            const MeReal x, const MeReal y, const MeReal z);
MEPUBLIC
void              MEAPI MdtContactSetNormal(const MdtContactID c,
                            const MeReal x, const MeReal y, const MeReal z);
MEPUBLIC
void              MEAPI MdtContactSetPenetration(const MdtContactID c, const MeReal p);
MEPUBLIC
void              MEAPI MdtContactSetDirection(const MdtContactID c,
                            const MeReal x, const MeReal y, const MeReal z);
MEPUBLIC
void              MEAPI MdtContactSetWorldVelocity(const MdtContactID c,
                            const MeReal x, const MeReal y, const MeReal z);
MEPUBLIC
void              MEAPI MdtContactSetParams(const MdtContactID c,
                            const MdtContactParamsID p);

MEPUBLIC
void              MEAPI MdtContactSetDimensions(const MdtContactID c, MeI32 dims); 

MEPUBLIC
void              MEAPI MdtContactSetNext(const MdtContactID c, const MdtContactID nc); /* DEPRECATED!! */

MEPUBLIC
void              MEAPI MdtContactSetUserData(const MdtContactID c, void *data); /* DEPRECATED!! */

#ifdef __cplusplus
}
#endif


#endif
